<?php

namespace NAPPS\Controllers;

use NAPPS\Contracts\IController;

class SmartBannerController implements IController {

    public function registerRoutes() {

        /**
        *   Use third party woocommerce authentication (api keys)
        *   to secure smart banner info post request   
        *
        *   @see https://github.com/woocommerce/woocommerce/blob/3611d4643791bad87a0d3e6e73e031bb80447417/plugins/woocommerce/includes/class-wc-rest-authentication.php#L65
        */
        register_rest_route(
            'wc-',
            'smartbanner',
            array(
                'methods'  => 'POST',
                'callback' => array( $this, 'smartbanner_info' ),
                'permission_callback' => array( $this, 'check_permission' ),
            )
        );

        register_rest_route(
            NAPPS_REST_PREFIX,
            'manifest',
            array(
                'methods'  => 'GET',
                'callback' => array( $this, 'manifest' ),
                'permission_callback' => '__return_true',
            )
        );
    }

    public function check_permission() {
        $user = get_current_user_id();
        return $user && current_user_can("manage_woocommerce");

    }

    /**
     * REST api to return manifest for our native application install
     *
     * @param  \WP_REST_Request $request
     * @return void
     */
    public function manifest( $request ) {

        $manifest = get_transient("napps_smartbanner_manifest");
        if(!$manifest) {
            $manifest = $this->generateManifestFromOption();
        }

        wp_send_json($manifest);
    }


    /**
     * Auth (admin) post in order to update smartbanner info when application goes live
     *
     * @param  \WP_REST_Request $request
     * @return \WP_REST_Response
     */
    public function smartbanner_info( $request ) {
        
        $apple_app_url    = $request->get_param( 'appleAppUrl' );
        $android_app_url    = $request->get_param( 'androidAppUrl' );
        $app_title    = $request->get_param( 'appTitle' );
        $app_desc    = $request->get_param( 'appDesc' );

        $nappsSmartAppBanner = array(
            "appTitle" => sanitize_text_field($app_title),
            "appDesc" => sanitize_text_field($app_desc),
            "appIcon" => '',
            "appleAppUrl" => sanitize_text_field($apple_app_url),
            "androidAppUrl" => sanitize_text_field($android_app_url),
        );

        if(isset($_FILES['icon']) && $this->validFile($_FILES['icon'])) {

            if ( ! function_exists( 'media_handle_upload' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/image.php' );
                require_once( ABSPATH . 'wp-admin/includes/file.php' );
                require_once( ABSPATH . 'wp-admin/includes/media.php' );
            }

            // Use the wordpress function to upload
            // 0 means the content is not associated with any other posts
            $uploaded = media_handle_upload('icon', 0);

            // Error checking using WP functions
            if(is_wp_error($uploaded) || !$uploaded) {
                return new \WP_REST_Response(null, 500);
            } else {
                $image = wp_get_attachment_image_url($uploaded);
                $nappsSmartAppBanner["appIcon"] = $image;
            }
        }

        update_option('nappsSmartAppBanner', $nappsSmartAppBanner);

        set_transient("napps_smartbanner_manifest", $this->generateManifestFromOption($nappsSmartAppBanner));

        return new \WP_REST_Response(array(), 200);
    }

    /**
     * Generate manifest from options
     *
     * @param  null|array $generateManifestFromOption
     * @return array
     */
    private function generateManifestFromOption($generateManifestFromOption = null) {
        
        $options = $generateManifestFromOption;
        if(!$options) {
            $options = get_option('nappsSmartAppBanner');
        }

        $manifest = array(
            "name" => "",
            "short_name" => "",
            "icons" => [
                array(
                    "src" => "launcher-icon-4x.png",
                    "sizes" => "192x192",
                    "type" => "image/png"
                )
            ],
            "prefer_related_applications" => true,
            "related_applications" => [
                "platform" => "play",
                "id" => "",
            ],
            "start_url" => "",
            "display" => "standalone"
        );

        if (!empty($options)) {
            $current_option = $options;

            if (!empty($current_option['appTitle'])) {
                $manifest["name"] = $current_option["appTitle"];
            }

            if (!empty($current_option['appDesc'])) {
                $manifest["short_name"] = $current_option["appDesc"];
            }

            if (array_key_exists("appIcon", $current_option) && !empty($current_option['appIcon'])) {
                for($i = 0; $i < count($manifest["icons"]); $i++) {
                    $manifest["icons"][$i]["src"] = $current_option["appIcon"];
                }
            }

            
            if (!empty($current_option['androidAppUrl'])) {
                $androidUrl = esc_url($current_option['androidAppUrl']);
                if(preg_match("/id=(.*)/", $androidUrl, $match)) {
                    $androidId = $match[1];
                    $manifest["related_applications"]["id"] = $androidId;
                }

                
            }
            $manifest["start_url"] = site_url();
        }

        return $manifest;
    }

    /**
     * Check if a file is valid
     *
     * @param  File $file
     * @return boolean
     */
    private function validFile($file) {
        if(!isset($file) || !isset($file['name'])) {
            return false;
        }

        $fileNameSplit = explode('.',$file['name']);
        $file_ext = end($fileNameSplit);

        // Check valid extension
        $file_ext = strtolower($file_ext);
        $extensions = array("jpeg","jpg","png");
        if(in_array($file_ext,$extensions) === false){
            return false;
        }

        return true;
    }
}